/* --------------------------------------------------------- */
/*	IHX2COM.c												 */
/*	Intel HEX File --> MSX .COM File Converter				 */
/* ========================================================= */
/*	Copyright(C)2006 t.hara									 */
/* --------------------------------------------------------- */

#include <stdio.h>
#include <ctype.h>

static unsigned char memory_image[ 65536 ];
static int max_address;

enum {
	TYPE_DATA	= 0,
	TYPE_END	= 1,

	TOP_ADDRESS	= 0x100,
};

/* --------------------------------------------------------- */
static int conv_hex( char c ) {

	if( isdigit(c)  ) return( c - '0' );
	if( isxdigit(c) ) return( toupper(c) - 'A' + 10 );
	return 0;
}

/* --------------------------------------------------------- */
static int get_one( const char **pp_line ) {
	int value;

	value = conv_hex( (*pp_line)[0] ) * 16 + conv_hex( (*pp_line)[1] );
	 *pp_line += 2;
	 return value;
}

/* --------------------------------------------------------- */
static int analize( const char *p_line ) {
	int length, offset, type, i;

	if( p_line[0] != ':' ) return 1;
	p_line++;

	length = get_one( &p_line );
	offset = get_one( &p_line );
	offset = offset * 256 + get_one( &p_line );
	type   = get_one( &p_line );

	switch( type ) {
	case TYPE_DATA:
		for( i = 0; i < length; i++ ) {
			memory_image[ offset + i ] = get_one( &p_line );
			if( max_address < (offset + i) ) max_address = offset + i;
		}
		break;
	case TYPE_END:
		printf( "Completed.\n" );
		return 0;
	default:
		fprintf( stderr, "ERR: Unknown data record type.\n" );
		return 0;
	}
	return 1;
}

/* --------------------------------------------------------- */
static int ihx2com( const char *p_com, const char *p_ihx ) {
	FILE *p_com_file;
	FILE *p_ihx_file;
	char s_line_buffer[ 256 ];

	p_ihx_file = fopen( p_ihx, "r" );
	if( p_ihx_file == NULL ) {
		fprintf( stderr, "ERR: Cannot open %s.\n", p_ihx );
		return 2;
	}

	p_com_file = fopen( p_com, "wb" );
	if( p_com_file == NULL ) {
		fclose( p_ihx_file );
		fprintf( stderr, "ERR: Cannot create %s.\n", p_com );
		return 3;
	}

	max_address = TOP_ADDRESS;

	while( ! feof( p_com_file ) ) {
		if( fgets( s_line_buffer, sizeof(s_line_buffer), p_ihx_file ) == 0 ) break;
		if( ! analize( s_line_buffer ) ) break;
	}

	fwrite( memory_image + TOP_ADDRESS, 1, max_address - TOP_ADDRESS + 1, p_com_file );

	fclose( p_com_file );
	fclose( p_ihx_file );
	return 0;
}

/* --------------------------------------------------------- */
static void usage( const char *p_cmd ) {

	fprintf( stderr, "Usage> %s <ihx file> <com file>\n", p_cmd );
}

/* --------------------------------------------------------- */
int main( int argc, char *argv[] ) {

	printf( "IHX2COM Copyright(C)2006 t.hara\n" );
	if( argc < 3 ) {
		usage( argv[0] );
		return 1;
	}

	return ihx2com( argv[2], argv[1] );
}
