/* --------------------------------------------------------- */
/*	MSX Functions											 */
/* ========================================================= */
/*	2006/11/25	t.hara										 */
/* --------------------------------------------------------- */

#ifndef __MSX_H__
#define __MSX_H__

/* --------------------------------------------------------- */
/* I/O port functions										 */
void outport( unsigned char port, unsigned char data );
unsigned char inport( unsigned char port );
void outports( unsigned char port, unsigned char *p_data, unsigned char count );

/* --------------------------------------------------------- */
/*	VDP functions											 */
void vdp_write( unsigned char vdpreg, unsigned char data );
unsigned char vdp_status( unsigned char vdpreg );
void vdp_write_ni( unsigned char vdpreg, unsigned char data );
unsigned char vdp_status_ni( unsigned char vdpreg );

#define is_vsyuc()	((vdp_status_ni( 0 ) & 0x80) != 0)
#define is_hsyuc()	((vdp_status_ni( 1 ) & 0x01) != 0)

/* --------------------------------------------------------- */
/*	PSG functions											 */
void psg_write( unsigned char psgreg, unsigned char data );
unsigned char psg_read( unsigned char psgreg );

/* --------------------------------------------------------- */
/*	File functions											 */
typedef struct {
	unsigned char	drive_no;
	unsigned char	name[8];
	unsigned char	ext[3];
	unsigned int	current_block;
	unsigned int	record_size;
	unsigned long	file_size;
	unsigned int	date;
	unsigned int	time;
	unsigned char	device_id;
	unsigned char	directory_location;
	unsigned int	start_cluster_no;
	unsigned int	last_access_cluster_no;
	unsigned int	cluster_offset;
	unsigned char	current_record;
	unsigned long	random_record;
} FCB;

typedef struct {
	unsigned char	name[8];
	unsigned char	ext[3];
	unsigned char	attribute;
	unsigned char	undel_char;			/* MSX-DOS2 */
	unsigned char	reserve[9];
	unsigned int	time;
	unsigned int	date;
	unsigned int	start_cluster_no;
	unsigned long	file_size;
} FCB_DIR;

typedef struct {
	unsigned char	drive_no;
	FCB_DIR			dirinfo;
} FCB_FIND;

/* return code */
#define FCB_SUCCESS	0x00

/* FCB_DIR::attribute */
#define	FCB_ATTR_READONLY	0x01		/* MSX-DOS2 */
#define FCB_ATTR_HIDDEN		0x02
#define	FCB_ATTR_SYSTEM		0x04		/* MSX-DOS2 */
#define FCB_ATTR_VOLUME		0x08		/* MSX-DOS2 */
#define	FCB_ATTR_DIR		0x10		/* MSX-DOS2 */
#define	FCB_ATTR_ARCHIVE	0x20		/* MSX-DOS2 */

unsigned char fcb_open( FCB *p_fcb );
unsigned char fcb_create( FCB *p_fcb );
unsigned char fcb_close( FCB *p_fcb );
unsigned int fcb_read( FCB *p_fcb, void *p_buffer, int size );
unsigned char fcb_write( FCB *p_fcb, const void *p_buffer, int size );
unsigned char fcb_find_first( FCB *p_fcb, FCB_FIND *p_result );
unsigned char fcb_find_next( FCB_FIND *p_result );

/* --------------------------------------------------------- */
void screen( unsigned char mode );

void vpoke_first( unsigned int address );
void vpoke( unsigned int address, unsigned char data );
#define vpoke_next( data )	outport( 0x98, (data) )

void vpeek_first( unsigned int address );
unsigned char vpeek( unsigned int address );
#define vpeek_next()	inport( 0x98 )

#define poke( address, data )	( *( (volatile unsigned char*)(address) ) = ( (unsigned char)(data) ) )
#define pokew( address, data )	( *( (volatile unsigned int*)(address) ) = ( (unsigned int)(data) ) )

#define peek( address )			( *( (volatile unsigned char*)(address) ) )
#define peekw( address )		( *( (volatile unsigned int*)(address) ) )

void set_display_page( unsigned char disp_page );
void set_active_page( unsigned char act_page );

void set_sprite_pattern( unsigned char pattern_no, unsigned char* p_pattern );
void put_sprite( unsigned char sprite_no, unsigned char x, unsigned char y, 
		unsigned char pattern_no, unsigned char ec_color );

#define	enable_intr()	{ _asm ei; _endasm; }
#define	disable_intr()	{ _asm di; _endasm; }

void set_interrupt_handler( char (*p_handler)( void ) );

#endif
