/* --------------------------------------------------------- */
/*	VDP BLANKING TEST FOR V9958								 */
/* ========================================================= */
/*	2008/05/04	t.hara										 */
/* --------------------------------------------------------- */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <msx/msx.h>

#define TEST_TIMES	10

/* --------------------------------------------------------- */
static int get_h_blanking_time( void ) {
	unsigned char start_time, end_time;
	int diff;

	disable_intr();
	vdp_write_ni( 15, 2 );					/* VDP status register S#2 read */
	while( (inport(0x99) & 0x20) != 0 );	/* wait HR=1 */
	start_time = inport(0xE6);
	while( (inport(0x99) & 0x20) == 0 );	/* wait HR=0 */
	end_time = inport(0xE6);
	vdp_write_ni( 15, 0 );
	enable_intr();

	diff = (int)end_time - (int)start_time;
	if( diff < 0 ) diff += 256;
	return diff;
}

/* --------------------------------------------------------- */
static int get_v_blanking_time( void ) {
	unsigned char start_time, end_time;
	int diff;

	disable_intr();
	vdp_write_ni( 15, 2 );					/* VDP status register S#2 read */
	while( (inport(0x99) & 0x40) != 0 );	/* wait VR=1 */
	start_time = inport(0xE6);
	while( (inport(0x99) & 0x40) == 0 );	/* wait VR=0 */
	end_time = inport(0xE6);
	vdp_write_ni( 15, 0 );
	enable_intr();

	diff = (int)end_time - (int)start_time;
	if( diff < 0 ) diff += 256;
	return diff;
}

/* --------------------------------------------------------- */
int main( void ) {
	int i;
	int hd_time[ TEST_TIMES ];
	int vd_time[ TEST_TIMES ];

	for( i = 0; i < TEST_TIMES; i++ ) {
		hd_time[i] = get_h_blanking_time();
	}
	for( i = 0; i < TEST_TIMES; i++ ) {
		vd_time[i] = get_v_blanking_time();
	}

	for( i = 0; i < TEST_TIMES; i++ ) {
		printf( "HD[%d] = %d\n", i + 1, hd_time[i] );
	}
	for( i = 0; i < TEST_TIMES; i++ ) {
		printf( "VD[%d] = %d\n", i + 1, vd_time[i] );
	}
	return 0;
}
