/* --------------------------------------------------------- */
/*	1chipMSX SCC test										 */
/* ========================================================= */
/*	2007/01/18	t.hara										 */
/* --------------------------------------------------------- */

#include <stdio.h>
#include <msx/msx.h>

/* --------------------------------------------------------- */
int is_ram( int address ) {
	unsigned char mem_back;

	mem_back = peek( address );
	poke( address, ~mem_back );
	if( peek( address ) == mem_back ) {
		return 0;	/* not RAM */
	}
	return 1;	/* RAM */
}

/* --------------------------------------------------------- */
void check_8000h_writable( int line ) {
	const char *p_message[] = { "8000h != RAM (%d)\n", "8000h == RAM (%d)\n" };

	printf( p_message[ is_ram( 0x8000 ) ], line );
}

/* --------------------------------------------------------- */
int get_scc_type( int slot ) {
	char ppi_port_a;
	char mem_back;

	/*	XbgZN^obNAbv */
	ppi_port_a = inport( 0xA8 );

	/*	page2 w̃Xbgɂ */
	outport( 0xA8, (ppi_port_a & 0xCF) | (slot << 4) );

	poke( 0xBFFE, 0x20 );
	check_8000h_writable( __LINE__ );
	poke( 0xB000, 0x00 );
	check_8000h_writable( __LINE__ );

	mem_back = peek( 0xB800 );
	poke( 0xB800, ~mem_back );
	check_8000h_writable( __LINE__ );
	if( is_ram( 0xB800 ) ) {
		outport( 0xA8, ppi_port_a );
		return 0;	/* not SCC */
	}

	poke( 0xB000, 0x80 );
	check_8000h_writable( __LINE__ );
	if( is_ram( 0xB800 ) ) {
		poke( 0xBFFE, 0x00 );
		outport( 0xA8, ppi_port_a );
		return 1;	/* SCC (Xib`[) */
	}

	poke( 0xBFFE, 0x00 );
	check_8000h_writable( __LINE__ );
	poke( 0x9000, 0x00 );
	check_8000h_writable( __LINE__ );

	if( is_ram( 0x9800 ) ) {
		outport( 0xA8, ppi_port_a );
		return 0;	/* not SCC */
	}

	poke( 0x9000, 0x3F );
	check_8000h_writable( __LINE__ );
	if( !is_ram( 0x9800 ) ) {
		outport( 0xA8, ppi_port_a );
		return 0;	/* not SCC */
	}

	poke( 0x9000, 0x00 );
	check_8000h_writable( __LINE__ );

	outport( 0xA8, ppi_port_a );
	return 2;	/* SCC (K) */
}

/* --------------------------------------------------------- */
int main( void ) {
	const char *p_type[] = {
		"not SCC", "SCC(SNATCHER)", "SCC(MegaROM)"
	};

	printf( "SLOT1: %s\n", p_type[ get_scc_type( 1 ) ] );
	printf( "SLOT2: %s\n", p_type[ get_scc_type( 2 ) ] );
	return 0;
}
