/* --------------------------------------------------------- */
/*	FILE test												 */
/* ========================================================= */
/*	2006/11/30	t.hara										 */
/* --------------------------------------------------------- */

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <msx/msx.h>

static FCB file;
static FCB_FIND find;

static const char s_test_data[] =
	"Hello, World!!!\xD\xA"
	"\xD\xA"
	"H   H EEEE L    L     OO  !!\xD\xA"
	"H   H E    L    L    O  O !!\xD\xA"
	"HHHHH EEE  L    L    O  O !!\xD\xA"
	"H   H E    L    L    O  O   \xD\xA"
	"H   H EEEE LLLL LLLL  OO  !!\xD\xA";

/* --------------------------------------------------------- */
static void set_name( FCB *p_fcb, const char *p_name ) {
	int i, j;

	memset( p_fcb, 0, sizeof(FCB) );
	for( i = 0; i < 11; i++ ) {
		p_fcb->name[i] = ' ';
	}
	for( i = 0; (i < 8) && (p_name[i] != 0) && (p_name[i] != '.'); i++ ) {
		p_fcb->name[i] = toupper( p_name[i] );
	}
	if( p_name[i] == '.' ) {
		i++;
		for( j = 0; (j < 3) && (p_name[i + j] != 0) && (p_name[i + j] != '.'); j++ ) {
			p_fcb->ext[j] = toupper( p_name[i + j] );
		}
	}
}

/* --------------------------------------------------------- */
static void put_dirinfo( FCB_DIR *p_di ) {
	int i;

	for( i = 0; i < 8; i++ ) {
		putchar( p_di->name[i] );
	}
	putchar( '.' );
	for( i = 0; i < 3; i++ ) {
		putchar( p_di->ext[i] );
	}
	putchar( '\n' );
}

/* --------------------------------------------------------- */
int main( void ) {
	int r;
	char buffer[ 33 ];

	/* read test */
	file.drive_no = 0;
	file.current_block = 0;
	set_name( &file, "TEST.TXT" );
	if( fcb_create( &file ) != FCB_SUCCESS ) {
		puts( "Failed: fcb_create()" );
		return 0;
	}
	r = strlen( s_test_data );
	printf( "strlen() = %d\n", r );
	r = fcb_write( &file, s_test_data, r );
	printf( "fcb_write() = %d\n", r );
	if( fcb_close( &file ) != FCB_SUCCESS ) {
		puts( "Failed: fcb_close()" );
		return 0;
	}
	puts( "---------------" );

	/* write test */
	file.drive_no = 0;
	file.current_block = 0;
	set_name( &file, "TEST.TXT" );
	if( fcb_open( &file ) != FCB_SUCCESS ) {
		puts( "Failed: fcb_open()" );
		return 0;
	}
	for( ;; ) {
		r = fcb_read( &file, buffer, 32 );
		if( r == 0 ) break;
		buffer[ r ] = 0;
		printf( "%s", buffer );
	}
	if( fcb_close( &file ) != FCB_SUCCESS ) {
		puts( "Failed: fcb_close()" );
		return 0;
	}
	puts( "---------------" );

	/* find test */
	file.drive_no = 0;
	set_name( &file, "????????.???" );
	if( fcb_find_first( &file, &find ) != FCB_SUCCESS ) {
		puts( "File not found." );
	}
	else {
		put_dirinfo( &find.dirinfo );
		while( fcb_find_next( &find ) == FCB_SUCCESS ) {
			put_dirinfo( &find.dirinfo );
		}
	}

	puts( "Success of all." );
	return 0;
}
